import { Button, Table, Modal } from "antd";
import { api } from "@nice/client";
import { useMainContext } from "../layout/MainProvider";
import dayjs from "dayjs";
import { useState } from "react";


// 定义 editRecord 的类型接口
interface EditRecord {
  id: string;
  staffId?: string;
  staffName?: string;
  position?: string;
  trainContentId?: string;
  content?: string;
  isPresent: boolean;
}

export default function DailyTable() {
  const { form, setVisible, searchValue, formValue } = useMainContext();
  const [editingRecord, setEditingRecord] = useState<EditRecord | null>(null);
  const { data: trainSituations, isLoading, refetch } =
    api.trainSituation.findMany.useQuery({
      where: {
        ...(searchValue ? {
          staff: {
            showname: {
              contains: searchValue
            }
          }
        } : {})
      },
      include: {
        staff: {
          include: {
            position: true,
            department: true
          }
        },
        trainContent: true,

      }
    });
  // console.log('data', trainSituations);

  // 基于日期过滤数据（如果后端不支持日期过滤）
  const filteredTrainSituations = trainSituations?.filter(situation => {
    // 检查是否有日期条件
    if (!formValue.date) return true;

    // 将记录日期转换为YYYY-MM-DD格式
    const situationDate = dayjs(situation.createdAt).format('YYYY-MM-DD');
    return situationDate === formValue.date;
  }) || [];


  const columns = [
    {
      title: "人员姓名",
      dataIndex: ["staff", "showname"],
      key: "showname",
      render: (name, record) => name || record.staff?.showname || "未知"
    },
    {
      title: "单位",
      dataIndex: ["staff", "department", "name"],
      key: "dept",
      render: (name) => name || "未知单位"
    },
    {
      title: "职务",
      dataIndex: ["staff", "position", "type"],
      key: "position",
      render: (type, record) => type || "无职务"
    },
    {
      title: "训练内容",
      dataIndex: ["trainContent", "title"],
      key: "trainContent",
      render: (title, record) => title || record.value || "未知内容"
    },
    {
      title: "在位情况",
      dataIndex: "alreadyTrainTime",
      key: "isPresent",
      render: (alreadyTrainTime) => alreadyTrainTime > 0 ? "在位" : "不在位"
    },
    {
      title: "操作",
      key: "action",
      render: (_, record) => (
        <div className="flex space-x-2">
          <Button
            type="primary"
            key={record.id}
            onClick={() => handleEdit(record)}
          >
            编辑
          </Button>
        </div>
      ),
    }
  ];

  const handleEdit = (record: any) => {
    console.log(record);
    
    const editRecord: EditRecord = {
      id: record.id,
      staffId: record.staff?.id,
      staffName: record.staff?.showname,
      position: record.staff?.position?.type,
      trainContentId: record.trainContent?.id,
      content: record.value,
      isPresent: record.alreadyTrainTime > 0,
    };

    setEditingRecord(editRecord);
    form.setFieldsValue(editRecord);
    setVisible(true);
    refetch();
  };

  return (
    <>
      {isLoading ? (
        <div>加载中...</div>
      ) : (
        <Table
          columns={columns}
          dataSource={filteredTrainSituations}
          rowKey="id"
          tableLayout="fixed"
          pagination={{
            position: ["bottomCenter"],
            className: "flex justify-center mt-4",
            pageSize: 10,
          }}
        />
      )}
    </>
  );
} 